# Generated by Django 5.0.3 on 2024-04-24 12:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0018_instrument_instrument_level_index"),
    ]

    operations = [
        migrations.AlterField(
            model_name="controversy",
            name="source",
            field=models.TextField(blank=True, null=True, verbose_name="Source"),
        ),
    ]
